<?php
namespace Zeedhi\DevOps\Util\Events\Translator\Audit;

use Doctrine\Common\EventSubscriber;
use Doctrine\ORM\Event\LifecycleEventArgs;
use Doctrine\ORM\Events;

class Subscriber  implements EventSubscriber{

    /** @var Environment */
    protected $environment;

    function __construct($environment){
        $this->environment = $environment;
    }

    public function prePersist(LifecycleEventArgs $args){
        $entity = $args->getEntity();
        if($entity instanceof AuditAble){
            $currentUserId = $this->environment->getCurrentUserId();
            if($currentUserId){
                $entity->setCreatedBy($currentUserId);
                $entity->setUpdatedBy($currentUserId);
            }

            $currentTime = $this->environment->getCurrentTime();
            $entity->setCreated($currentTime);
            $entity->setUpdated($currentTime);
        }
    }

    public function preUpdate(LifecycleEventArgs $args){
        $entity = $args->getEntity();
        if($entity instanceof AuditAble){
            $currentUserId = $this->environment->getCurrentUserId();

            if($currentUserId){
                $entity->setUpdatedBy($currentUserId);
            }

            $currentTime = $this->environment->getCurrentTime();
            $entity->setUpdated($currentTime);
        }
    }


    /**
     * Returns an array of events this subscriber wants to listen to.
     *
     * @return array
     */
    public function getSubscribedEvents(){
        return array(
            Events::prePersist,
            Events::preUpdate,
        );
    }
}